#include <Arduino.h>
#include "message-interpreter.h"
#include <SoftwareSerial.h>
void setup();
void loop();
void waitAndReadWifiSerial(int32_t ms);
void relayEn1(bool flag);
void relayEn2(bool flag);
void relayEn3(bool flag);
void relayEn4(bool flag);
#line 1 "src/sketch.ino"
// Author: Karl Zeilhofer, www.zeilhofer.co.at
// Creation date: 2.12.2014
// Documentation: http://www.zeilhofer.co.at/wiki/doku.php?id=esp8266_wifi_modules


/* TODO:
	implement a simple CLI for the USB-Uart, so we can set different constants:
		and save them into the EEPROM

		WiFi SSID
		WiFi key
		server name
		server file path

	TODO: Test the new interpreter keywords (SOF, EOF, Power)
*/



//#include "message-interpreter.h"
//#include <SoftwareSerial.h>

uint32_t time_current=0;
#define SERVER_NAME "www.zeilhofer.co.at"
#define SERVER_PORT 80
#define SERVER_PORT_STR "80"
#define SERVER_FILE_PATH "/lackergraben-heating/simple-config.txt"
#define STR_MAX_LEN 150
#define BUF_MAX_LEN 500

// pin-definitions:
#define PinRx 3
#define PinTx 2
#define PinRel_1 4
#define PinRel_2 5
#define PinRel_3 6
#define PinRel_4 7


// Wifi-Module is a SoftwareSerial
SoftwareSerial wifiSerial(PinRx, PinTx); // RX, TX

// the setup routine runs once when you press reset:
void setup() {
	// initialize serial communication at 9600 bits per second:
	Serial.begin(57600); // USB-serial, used for configuring and debugging
	wifiSerial.begin(9600); 

	pinMode(PinRel_1, OUTPUT);
	pinMode(PinRel_2, OUTPUT);
	pinMode(PinRel_3, OUTPUT);
	pinMode(PinRel_4, OUTPUT);


	relayEn1(false);
	relayEn2(false);
	relayEn3(false);
	relayEn4(false);

	// enable each relay for one second just for testing
	relayEn1(true);
	delay(1000);
	relayEn1(false);
	relayEn2(true);
	delay(1000);
	relayEn2(false);
	relayEn3(true);
	delay(1000);
	relayEn3(false);
	relayEn4(true);
	delay(1000);
	relayEn4(false);
}

int counter=0;
char str[STR_MAX_LEN+1]; // general purpose string
char inBuffer[BUF_MAX_LEN+1]={0}; 

void loop(){

	static uint32_t uptime_lastCheck=0;
	static bool firstRun = true;

	delay(1000); //loop only once per second

	// for switching scheme, refer to the schematic V1.1 of the 
	// "Uhrenraumthermostat RAM382F-20 from Windhager"
	// reverse engineered from Mechatronik Karl Zeilhofer
	int power = MI_currentHeatingRequirement();
	Serial.print("# power = ");
	Serial.println(power, DEC);

	switch(power){
	case 0: // off
		relayEn1(true);
		relayEn2(false);
		break;
	case 1: // low power
		relayEn1(false);
		relayEn2(false);
		break;
	case 2: // high power
		relayEn1(false);
		relayEn2(true);
		break;
	}


	// check for overflow of millis(), which returns the time in ms since power up. 
	static uint32_t lastMillis = 0;
	if(millis() < lastMillis){ // overflow occured
		uptime_lastCheck = 0;
	}

	// check once in a minute for new input data on the defined server
	if(millis()/1000 >= uptime_lastCheck+30 || firstRun){
		uptime_lastCheck = millis()/1000;
		firstRun = false;

		wifiSerial.print("AT+RST\r\n");
		waitAndReadWifiSerial(1500); // wait some seconds for reboot 
		//wifiSerial.print("AT+CWJAP=\"repeater\",\"fvo-rp27\"\r\n"); // join access point
		wifiSerial.print("AT+CWJAP=\"MKZ\",\"BE662F3BC9\"\r\n"); // join access point
		waitAndReadWifiSerial(7000);

		// open TCP connection:
		wifiSerial.print("AT+CIPSTART=\"TCP\",\"");
		wifiSerial.print(SERVER_NAME);
		wifiSerial.print("\",");
		wifiSerial.print(SERVER_PORT);
		wifiSerial.print("\r\n");
		waitAndReadWifiSerial(200);

		// send GET request:
		wifiSerial.print("AT+CIPSEND=");
		wifiSerial.print(4+strlen(SERVER_FILE_PATH)+9+2, DEC);
		wifiSerial.print("\r\n");
		waitAndReadWifiSerial(100);
		wifiSerial.print("GET ");
		wifiSerial.print(SERVER_FILE_PATH);
		wifiSerial.print(" HTTP/1.1\r\n");
		waitAndReadWifiSerial(100);

		wifiSerial.print("AT+CIPSEND=");
		wifiSerial.print(6+strlen(SERVER_NAME)+1+strlen(SERVER_PORT_STR)+2, DEC);
		wifiSerial.print("\r\n");
		waitAndReadWifiSerial(100);
		wifiSerial.print("Host: ");
		wifiSerial.print(SERVER_NAME);
		wifiSerial.print(":");
		wifiSerial.print(SERVER_PORT_STR);
		wifiSerial.print("\r\n");
		waitAndReadWifiSerial(100);

		wifiSerial.print("AT+CIPSEND=2\r\n");
		waitAndReadWifiSerial(100);
		wifiSerial.print("\r\n"); // empty line for HTTP-Get end-of-header


		int32_t start = millis();

		waitAndReadWifiSerial(1000); // receive response from server...

		int n=0;
		int i=0;
		while(inBuffer[n] && n<=BUF_MAX_LEN-1)
		{
			str[i] = inBuffer[n];

			if(str[i] == '\n'){
				str[i]='\0'; // terminate line on line-feed character
				MI_interpretHttpGetResponse(str);
				i=0; // reset index for next line. 
			}else if(str[i] == '\r'){
				// do nothing here, ignore CR
			}else{ // increment i
				i++; 
				if(i >= STR_MAX_LEN){
					// too long line
					// terminate it, and try to interpret it. 
					str[STR_MAX_LEN]=0;
					//MI_interpretHttpGetResponse(str);
					i=0;
				}
			}

			n++;
		}
	}


	// forward all debug-inputs to the wifi-module
	if(Serial.available()){
		wifiSerial.write(Serial.read());
	}
}


// waits ms milliseconds, and polls for incoming
// data on wifiSerial and forwards it to the Serial
// the data is also stored in inBuffer[] and zero-terminated
void waitAndReadWifiSerial(int32_t ms)
{
	int32_t start = millis();
	int i=0;
	int32_t tLastData = 0;

	while(millis() < start+ms)
	{
		if(wifiSerial.available() && i<=BUF_MAX_LEN-1){
			inBuffer[i++] = wifiSerial.read(); // for speed reason, read to a buffer
			tLastData = millis();
		}	
	}
	if(i >= BUF_MAX_LEN-1)
	{
		Serial.println("$ BUF_MAX_LEN exceeded");
	}
	inBuffer[i]=0;

	// print info: 
	// $ Rx-Duration: 41/100ms; received 23 bytes
	Serial.print("$ Rx-Duration: ");
	Serial.print(tLastData-start, DEC);
	Serial.print("/");
	Serial.print(ms, DEC);
	Serial.print("ms; received ");
	Serial.print(i, DEC);
	Serial.println(" bytes:");

	Serial.write(inBuffer);
	Serial.println();
	Serial.flush(); // transmit to USB after receiving complete message!
}


// K1 and K2 must not be enabled simultaneously!
void relayEn1(bool flag)
{
	if(flag){
		digitalWrite(PinRel_1, LOW); // active low
		relayEn2(false); // make sure, K2 is disabled. 
	}else{
		digitalWrite(PinRel_1, HIGH);
	}
}
void relayEn2(bool flag)
{
	if(flag){
		digitalWrite(PinRel_2, LOW); // active low
		relayEn1(false); // make sure, K1 is disabled. 
	}else{
		digitalWrite(PinRel_2, HIGH);
	}
}

// K3 and K4 are not used (12.12.2014)
void relayEn3(bool flag)
{
	if(flag){
		digitalWrite(PinRel_3, LOW); // active low
	}else{
		digitalWrite(PinRel_3, HIGH);
	}
}
void relayEn4(bool flag)
{
	if(flag){
		digitalWrite(PinRel_4, LOW); // active low
	}else{
		digitalWrite(PinRel_4, HIGH);
	}
}


