

/* TODO:
	use the Time.h library (must be downloaded)
*/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <Time.h>

#include "message-interpreter.h"


static time_t time_fileModified = 1; // holds the time, when the input file on the server was modified last time. 
static bool fileWasModified = false;
static time_t time_nextOn=1;
static time_t time_nextOff=1;
static bool fileOpened = false; 
static int power = 0;

static int GMT_offset = +1; // default value for central europe time (winter time)
	// can be redefined via the simple-config.txt

struct MyTime
{
	int hour;   // 0...23
	int minute; // 0...59
	int second; // 0...59
};

struct MyTime MyTime_On;
struct MyTime MyTime_Off;


// this functions gets a pointer to a line, which must be zero-terminated (but without any \n or \r)
void MI_interpretHttpGetResponse(const char* line)
{
	// Serial.print("$ Parsing line: ");
	// Serial.println(line);

	if(fileOpened == false) // look for SOF and HTTP-Header-keywords
	{
		if(myStrCompare(line, "SOF-SIMPLE-CONFIG")==0)
		{
			Serial.println("$ Start of File");
			fileOpened = true;
		}else if(myStrCompare(line, "Date: ")==0)
		{
			// Format Example: Tue, 02 Dec 2014 21:34:00 GMT
			// TODO:
			Serial.print("$ Server date-time = ");
			time_t t = dateTimeStrToUnixTime(line+6) + GMT_offset*3600; // Attention: GMT_offset could be wrong after power-up
			setTime(t); // set local time
			Serial.println(timeToStr(0));
		}else if(myStrCompare(line, "Last-Modified: ")==0)
		{
			// Format Example: Tue, 02 Dec 2014 21:33:22 GMT
			// TODO:
			Serial.print("$ last modified = ");
			time_t tMod = dateTimeStrToUnixTime(line+15); // copy to temporary time
			Serial.print(timeToStr(tMod));
			if(tMod != time_fileModified)
			{
				Serial.print(" (has changed from: ");
				Serial.print(timeToStr(time_fileModified));
				Serial.println(")");
				fileWasModified = true;
			}else
			{
				Serial.println(" (has NOT changed)");
				fileWasModified = false;
			}
			time_fileModified = tMod; // save time to global variable!
		}
	}else // file is open, check for file-key-words:
	{
		if(myStrCompare(line, "GMT-Time-Offset: ")==0) 
		{
			// TODO:
			Serial.print("$ GMT-Time-Offset = ");
			GMT_offset = strtol(line+strlen("GMT-Time-Offset: "), NULL, 10);
			Serial.println(GMT_offset, DEC);
		}else if(myStrCompare(line, "Time-On: ")==0)
		{
			// TODO:
			Serial.print("$ Time-On = ");
			char* charAfterNumber=NULL;
			MyTime_On.hour = strtol(line+strlen("Time-On: "), &charAfterNumber, 10);
			MyTime_On.minute = strtol(charAfterNumber+1, &charAfterNumber, 10);
			MyTime_On.second = 0;
			printMyTime(MyTime_On);
			Serial.println();
		}else if(myStrCompare(line, "Time-Off: ")==0)
		{
			// TODO:
			Serial.print("$ Time-Off = ");
			char* charAfterNumber=NULL;
			MyTime_Off.hour = strtol(line+strlen("Time-Off: "), &charAfterNumber, 10);
			MyTime_Off.minute = strtol(charAfterNumber+1, &charAfterNumber, 10);
			MyTime_Off.second = 0;
			printMyTime(MyTime_Off);
			Serial.println();
		}else if(myStrCompare(line, "Power-Requirement (0/1/2): ")==0)
		{
			// TODO:
			Serial.print("$ Power = ");
			char* charAfterNumber=NULL;
			power = strtol(line+strlen("Power-Requirement (0/1/2): "), NULL, 10);
			switch (power) {
			    case 0:
			      Serial.println("Off");
			      break;
			    case 1:
			      Serial.println("Low");
			      break;
			    default:
			      Serial.println("High");
			      power = 2;
			}
		}else if(myStrCompare(line, "EOF")==0)
		{
			Serial.println("$ End of File");
			fileOpened = false; // close file, --> skip interpreter for file-keywords
			updateNextControlTimes();
			Serial.print("$ Local Time: ");
			Serial.println(timeToStr(0));
		}
	}
}


// this must be called, when the whole simple-config-file was interpreted
void updateNextControlTimes()
{
	TimeElements ts;
	time_t t = time_fileModified;


	// generate date-time from fileModified-time (day) with the specified On- and Off-Times
	breakTime(t, ts);
	ts.Hour = MyTime_On.hour;
	ts.Minute = MyTime_On.minute;
	ts.Second = MyTime_On.second;
	time_nextOn = makeTime(ts);

	breakTime(t, ts);
	ts.Hour = MyTime_Off.hour;
	ts.Minute = MyTime_Off.minute;
	ts.Second = MyTime_Off.second;
	time_nextOff = makeTime(ts);

	// On-Time can be in the past, but the Off-Time has to be in the future (max. 24h)!
	if(time_nextOff <= t) // the off-time must be after the on-time
		time_nextOff += 24*3600; // add a whole day, so it is in the future. 
	if(time_nextOn >= time_nextOff)
		time_nextOn -= 24*3600; // subtract a whole day, so it is before the off-time.  

	// Example: On Mondy at 22:00 we change the times as follow:
	// 		On-Time = 21:00, Off-Time = 01:00
	// 		What we mean is: turn the heating on on Tuesday, 21:00
	//		and off on Wednesday 01:00, after midnight. 
	//
	//		With the calculations above we get this times
	//		time_nextOn = Monday, 21:00
	//		since this is in the past, we correct it to
	//					--> Tuesday, 21:00
	//		time_nextOff = Monday, 01:00
	//		since it is in the past, relative to the time_nextOn, we add 48h:
	// 					--> Wednesday 01:00

	Serial.print("$ On-Time: ");
	Serial.println(timeToStr(time_nextOn));
	Serial.print("$ Off-Time: ");
	Serial.println(timeToStr(time_nextOff));

}

// returns true, when the heating shold be on now
// returns false otherwise
// call this function periodically, and switch the output relais accordingly!
int MI_currentHeatingRequirement()
{
	time_t t = now();
	if(t >= time_nextOn && t < time_nextOff)
		return power;
	else
		return 0;
}

// if a long string starts with a short string, zero is returned. 
int myStrCompare(const char* str1, const char* str2)
{
	int len1 = strlen(str1);
	int len2 = strlen(str2);

	int minLen = len1<len2?len1:len2;
	if(minLen==0)
	{
		return -1; // return non-zero, when one of the strings has zero length. 
	}

	// Serial.println("minLen = ");
	// Serial.println(minLen, DEC);
	return strncmp(str1, str2, minLen);
}


// converts a date-time-string like this one:
// Tue, 02 Dec 2014 21:33:22 GMT
// to a unix-time
uint32_t dateTimeStrToUnixTime(const char * str)
{
	char monthStr[20]; // e.g. Dec
	char dateStr[20]; // day
	char timeStr[20]; // e.g. 21:33:22

	int day;
	int year;


	// skip weekday:
	char* spaceAddress=NULL; // for strtol
	char* nextStr=NULL;
	spaceAddress = strpbrk(str, " "); // find first space
	nextStr = spaceAddress+1; // skip weekday + space

	// read day (02):
	char* strAfterNumber = NULL;
	day = strtol(nextStr, &strAfterNumber, 10);
	nextStr = strAfterNumber+1; // skip space after 02
	// Serial.print("$ day = ");
	// Serial.println(day, DEC);


	// read short month name (Dec)
	strncpy(monthStr, nextStr, 3);
	monthStr[3]=0; // terminate string
	nextStr = nextStr + 4;
	// Serial.print("$ month = ");
	// Serial.println(monthStr);

	// read year (2014)
	year = strtol(nextStr, &strAfterNumber, 10);
	nextStr = strAfterNumber+1; // skip space
	// Serial.print("$ year = ");
	// Serial.println(year, DEC);

	// copy time-string (21:33:22)
	spaceAddress = strpbrk(nextStr, " "); // find first space after time-string
	int i = spaceAddress-nextStr;
	strncpy(timeStr, nextStr, i);
	timeStr[i]=0; // terminate string
	// Serial.print("$ timeStr = ");
	// Serial.println(timeStr);


	int sec;
	int min;
	int hour;

	hour = strtol(timeStr, &strAfterNumber, 10);
	min = strtol(strAfterNumber+1, &strAfterNumber, 10); // skip :
	sec = strtol(strAfterNumber+1, &strAfterNumber, 10); // skip :

	TimeElements ts; // Arduino-Struct (Time.h)
	ts.Second=sec;
	ts.Minute=min;
	ts.Hour=hour;

	ts.Day=day;
	ts.Year=year-1970; // !!! not 1900 as in time.h

	int month=0; // start with january
	char* monthNames[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul",
				"Aug", "Sep", "Oct", "Nov", "Dec"};
	for(month=0; month<12; month++)
	{
		//printf("test for %s\n", monthNames[month]);
		if(strncmp(monthStr, monthNames[month], 3) == 0)
		{
			ts.Month = month+1; // in Time.h 1-12 is used
			break;
		}
	}
	if(ts.Month >= 1 && ts.Month<=12)
	{
		// now we found the correct month
		return makeTime(ts); // Arduino-Function (non-ANSI-C!!!)
	}
	else
	{
		// we didn't find the month
		return 0;
	}
}

// return a pointer to a static allocated string
// this string will be overwritten, on the next call of timeToStr();
// in the format: 2004-06-14T23:34:30
// if t=0, the current date-time is returned.
char* timeToStr(time_t t)
{
	if(t==0) // then use current time
	{
		t = now(); // Arduino-Function (non-ANSI-C!!!)
	}

	TimeElements ts;
	breakTime(t, ts); // Arduino-Function (non-ANSI-C!!!)

	static char str[21];
	snprintf(str, 20, "%d-%02d-%02dT%02d:%02d:%02d", ts.Year+1970, ts.Month, ts.Day,
			ts.Hour, ts.Minute, ts.Second);
	return str;
}

void printMyTime(struct MyTime time)
{
	Serial.print(time.hour, DEC);
	Serial.print(":");
	
	if(time.minute < 10)
		Serial.print("0");
	Serial.print(time.minute, DEC);
	Serial.print(":");
	if(time.second < 10)
		Serial.print("0");
	Serial.print(time.second, DEC);
}